#ifndef _BUFFER_H_INCLUDED
#define _BUFFER_H_INCLUDED

/* QNT - buffer.h - File buffer class. */


#define WIN32_LEAN_AND_MEAN
#include <limits.h>
#include <windows.h>


class Buffer
{
    static char *enterstr[4];
    static int enterlen[4];
    class LineList;
    void lineimage( int, char *, LineList * );
    void scrimage( int, int, char * );
    class LineList
    {
        int vislen;
        int len;
        char *str;
        LineList *next;
        friend void Buffer::lineimage( int, char *, LineList * );
        friend void Buffer::scrimage( int, int, char * );
    public:
        LineList() { vislen = 0; len = 0; str = 0; next = 0; }
        LineList( int v, int l, char *s, LineList *n )
        { vislen = v; len = l; str = s; next = n; }
        ~LineList() { delete [] str; delete next; }
        int getvislen() { return vislen; }
        int getlen() { return len; }
        char *getbuf() { return str; }
        char *cdrbuf() { char *s = str; str = 0; return s; }
        LineList *cdr() { LineList *n = next; next = 0; delete this; return n; }
    } *startlines, *endlines;
    class CharList
    {
        char ch;
        signed char tab;
        CharList *next;
        friend void Buffer::scrimage( int, int, char * );
    public:
        CharList() { ch = 0; next = 0; tab = -1; }
        CharList( char c, CharList *cdr ) { ch = c; tab = 0; next = cdr; }
        CharList( char c, signed char t, CharList *n ) { ch = c; tab = t; next = n; }
        ~CharList() { delete next; }
        char getch() { return ch; }
        signed char gettab() { return tab; }
        CharList *cdr() { CharList *n = next; next = 0; delete this; return n; }
        int count( int * = 0 );
        void put( char * );
        void putrev( char * );
    } *startchars, *endchars;
    int row, col, rows;
    int drow, dcol, arow, acol;
    int ent, tabsize;
    char *filename;
    char *autosavename;
    BOOL prehilite, invhilite, invscreen, changed, modified, truemod, exitstatus;

    void build();
    void brk();
    BOOL literal( char );
    void exitline( int &, int &, char *& );
    void enterline( int, int, char *, int );
    char *seekcol( char *, int );

public:
    BOOL getexitstatus() { return exitstatus; }
    BOOL modifiedP() { return modified; }

/* buffer.cpp: */
    Buffer();
    ~Buffer();
    void postmap( BOOL );
    BOOL seekto( int, int );
    void prefloc( int, int );

/* file.cpp: */
    void setname( const char * );
    BOOL autosave();
    BOOL saveas( const char * );
    void refresh();

/*****************/
/* user callable */
/*****************/
/* Miscellaneous: */
    BOOL exit() { return exitstatus = TRUE; }

/* user.cpp: */
/* Text insertion: */
    BOOL litchar( char );
    BOOL enter();
/* Edit commands operating on single line: */
    BOOL charleft();
    BOOL charright();
    BOOL linebeg();
    BOOL lineend();
    BOOL wordleft();
    BOOL wordright();
    BOOL delcharleft();
    BOOL delcharright();
    BOOL dellinebeg();
    BOOL dellineend();
    BOOL delwordleft();
    BOOL delwordright();
/* Multiline versions of single line operations: */
    BOOL delcharleftwrap();
    BOOL delcharrightwrap();
    BOOL delcharrightwrapsel();
/* Edit commands spanning multiple lines: */
    BOOL lineup();
    BOOL linedown();
    BOOL pageup();
    BOOL pagedown();
    BOOL filebeg() { return seekto( 0, 0 ); }
    BOOL fileend() { return seekto( rows - 1, INT_MAX ); }
    BOOL delline();
/* Block commands: */
    BOOL delselect();
    BOOL paste();
    BOOL copy();
    BOOL cut();

/* file.cpp: */
    BOOL load();
    BOOL save();
};


#endif
