/* QNT -  display.cpp - Display class. */


#define WIN32_LEAN_AND_MEAN
#include <string.h>
#include <windows.h>
#include "buffer.h"
#include "config.h"
#include "display.h"


Display::Display()
{
    hConsoleOutput = GetStdHandle( STD_OUTPUT_HANDLE );
    CONSOLE_SCREEN_BUFFER_INFO csbi;
    GetConsoleScreenBufferInfo( hConsoleOutput, &csbi );
    rows = SHORT( csbi.dwSize.Y - 1 );
    cols = csbi.dwSize.X;
    saveattr = csbi.wAttributes;
    COORD coord;
    coord.Y = coord.X = 0;
    DWORD i;
    FillConsoleOutputCharacter( hConsoleOutput, ' ', (rows+1) * cols, coord, &i );
    FillConsoleOutputAttribute( hConsoleOutput, Config::gettextattr(),
        rows * cols, coord, &i );
    coord.Y = rows;
    FillConsoleOutputAttribute( hConsoleOutput, Config::getstatusattr(),
        cols, coord, &i );
}


Display::~Display()
{
    COORD coord;
    coord.Y = coord.X = 0;
    DWORD i;
    FillConsoleOutputCharacter( hConsoleOutput, ' ', (rows+1) * cols, coord, &i );
    FillConsoleOutputAttribute( hConsoleOutput, saveattr, (rows+1) * cols,
        coord, &i );
    SetConsoleCursorPosition( hConsoleOutput, coord );
}


void Display::redraw( Buffer *buffer, void (Buffer::*f)( int, int, char * ) )
{
    char *buf = new char[rows * cols];
    (buffer->*f)( rows, cols, buf );
    COORD coord;
    coord.Y = coord.X = 0;
    DWORD i;
    WriteConsoleOutputCharacter( hConsoleOutput, buf, rows * cols, coord, &i );
    delete [] buf;
}


void Display::hilite( SHORT startrow, SHORT startcol, SHORT endrow, SHORT endcol )
{
    COORD coord;
    coord.Y = coord.X = 0;
    DWORD i;
    FillConsoleOutputAttribute( hConsoleOutput, Config::gettextattr(),
        rows * cols, coord, &i );
    coord.Y = SHORT( min( max( startrow, 0 ), rows ) );
    coord.X = SHORT( min( max( startcol, 0 ), cols ) );
    DWORD cnt = (min( max( endrow, 0 ), rows ) - coord.Y) * cols
        + min( max( endcol, 0 ), cols ) - coord.X;
    FillConsoleOutputAttribute( hConsoleOutput, Config::gethiliteattr(), cnt,
        coord, &i );
}


void Display::drawstatus( char *str, SHORT y, SHORT x )
{
    char *buf = new char[cols];
    int len = strlen( str );
    if( len < cols )
    {
        memcpy( buf, str, len );
        memset( buf + len, ' ', cols - len );
    }
    else
        memcpy( buf, str, cols );
    COORD coord;
    coord.Y = rows;
    coord.X = 0;
    DWORD i;
    WriteConsoleOutputCharacter( hConsoleOutput, buf, cols, coord, &i );
    delete [] buf;
    coord.Y = y;
    coord.X = x;
    SetConsoleCursorPosition( hConsoleOutput, coord );
}


void Display::message( const char *str )
{
    char *buf = new char[cols];
    int len = strlen( str );
    if( len < cols )
    {
        memcpy( buf, str, len );
        memset( buf + len, ' ', cols - len );
    }
    else
        memcpy( buf, str, cols );
    COORD coord;
    coord.Y = rows;
    coord.X = 0;
    DWORD i;
    WriteConsoleOutputCharacter( hConsoleOutput, buf, cols, coord, &i );
    delete [] buf;
    coord.X = WORD( min( len, cols - 1 ) );
    SetConsoleCursorPosition( hConsoleOutput, coord );
}
