/* QNT - keyboard.cpp - Keyboard input class. */


#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "buffer.h"
#include "config.h"
#include "keyboard.h"


/***********************************************************************
    Public:
***********************************************************************/

Keyboard::Keyboard()
{
    hConsoleInput = GetStdHandle( STD_INPUT_HANDLE );
    GetConsoleMode( hConsoleInput, &fdwMode );
    SetConsoleMode( hConsoleInput, ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT );
    meta = FALSE;
    top = FALSE;
    altnum = 0;
    autosave = 0;
}


Keyboard::~Keyboard()
{
    SetConsoleMode( hConsoleInput, fdwMode );
}


BOOL Keyboard::emptyP()
{
    DWORD cnt;
    GetNumberOfConsoleInputEvents( hConsoleInput, &cnt );
    return !cnt;
}


void Keyboard::poll( Buffer *buffer )
{
    if( buffer->modifiedP() )
    {
        int waittime = autosave - GetTickCount();
        if( waittime <= 0
         || WaitForSingleObject( hConsoleInput, waittime ) == WAIT_TIMEOUT )
        {
            buffer->autosave();
            autosave = 0;
        }
    }
    else
        WaitForSingleObject( hConsoleInput, INFINITE );
    INPUT_RECORD ir;
    DWORD i;
    ReadConsoleInput( hConsoleInput, &ir, 1, &i );
    if( ir.EventType == KEY_EVENT )
    {
        KEY_EVENT_RECORD &ker = ir.Event.KeyEvent;
        if( ker.bKeyDown )
            while( ker.wRepeatCount-- )
                keydown( buffer, ker.wVirtualKeyCode, ker.uChar.AsciiChar,
                 ker.dwControlKeyState );
        else
            while( ker.wRepeatCount-- )
                keyup( buffer, ker.wVirtualKeyCode );
    }
    else if( ir.EventType == MOUSE_EVENT )
    {
        MOUSE_EVENT_RECORD &mer = ir.Event.MouseEvent;
        if( mer.dwButtonState & 1 )
        {
            buffer->prefloc( dwMousePosition.Y, dwMousePosition.X );
            if( !mer.dwEventFlags )
                buffer->postmap( FALSE );
            else if( mer.dwEventFlags == MOUSE_MOVED )
                buffer->postmap( TRUE );
        }
    }
    if( buffer->modifiedP() )
    {
        if( !autosave )
            autosave = GetTickCount() + Config::getautosaveinterval();
    }
    else
        autosave = 0;
}


BOOL Keyboard::yesnoP()
{
    char ch;
    DWORD i;
    do
    {
        ReadConsole( hConsoleInput, &ch, 1, &i, NULL );
        ch |= 0x20;
    } while( ch != 'y' && ch != 'n' );
    if( ch == 'y' )
        return TRUE;
    else
        return FALSE;
}


/***********************************************************************
    Private:
***********************************************************************/

void Keyboard::keydown( Buffer *buffer, WORD key, CHAR ch, DWORD ctrl )
{
    if( ctrl & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED)
     && (key >= VK_NUMPAD0 && key <= VK_NUMPAD9) )
        altnum = char( 10 * altnum + key - VK_NUMPAD0 );
    else
    {
        altnum = 0;
        switch( key )
        {
          case 91:
          case 92:
            meta = TRUE;
            break;
          case 93:
            top = TRUE;
            break;
          default:
            map( buffer, key, ch, ctrl );
        }
    }
}


void Keyboard::keyup( Buffer *buffer, WORD key )
{
    switch( key )
    {
      case VK_MENU:
        if( altnum )
        {
            buffer->litchar( altnum );
            altnum = 0;
        }
        break;
      case 91:
      case 92:
        meta = FALSE;
        break;
      case 93:
        top = FALSE;
        break;
    }
}


void Keyboard::map( Buffer *buffer, WORD key, CHAR ch, DWORD ctrl )
{
    BOOL shiftstate = !!(ctrl & SHIFT_PRESSED);
    int code = 0;
    if( ctrl & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED) )
        code |= Config::ALT;
    if( ctrl & (LEFT_CTRL_PRESSED | RIGHT_CTRL_PRESSED) )
        code |= Config::CTRL;
    if( meta )
        code |= Config::META;
    if( top )
        code |= Config::TOP;
    if( !Config::mapkey( buffer, Config::shiftxform( code ) | key ) )
    {
        if( ch )
        {
            buffer->litchar( ch );
            buffer->postmap( FALSE );
        }
    }
    else
        buffer->postmap( shiftstate );
}
